#!/bin/bash
set -eo pipefail

default_api_key_path=/var/opt/fortanix/em-agent/join-token
em_agent_conf_file=/etc/em-agent/em-agent.conf
install_enclave_os=true
default_attestation_type_path=/var/opt/fortanix/em-agent/attestation-type
TMP_DIR="/tmp/em-agent/"

. /etc/os-release
ubuntu_version=${VERSION_ID}
ubuntu_codename=${VERSION_CODENAME}

# Default values
dcap_attestation_supported=false
dcap_provider_library_installed=false
flc_supported=false
attestation_type=""

if [ "$EUID" -ne 0 ]; then
    run_as_root_msg="Please run this file as root"
    echo ${run_as_root_msg}
    exit 1
fi

function create_new_dir() {
    rm -rf $1
    mkdir -p $1
}

create_new_dir $TMP_DIR


function usage() {
    installer_usage="USAGE:    sudo bash $0 [--no-enclave-os] <join-token> --attestation-type=<attestation-type>"
    installer_options="OPTIONS:
	--no-enclave-os				 Do not install packages needed to run Enclave OS images
	--attestation-type=<attestation-type>	 Attestation type supported are EPID or DCAP."
    echo -e "${installer_usage}"
    echo -e "${installer_options}"
}

function append_line_if_not_exists() {
    grep -qxF "$1" "$2" || echo "$1" >> "$2"
}

function die() {
    echo "error - see log file $log_file for full details" >&3
    echo "Tail of the log file:" >&3
    tail -n 15 "$log_file" >&3
    exit 1
}

function yaml_processor_tool_installation() {
    apt-get update
    apt-get install -y python3-pip jq wget
    pip3 install yq
    echo "* Yaml processor tool yq installation successful... " >&3
}

function generate_sgx_detect_report() {
    yaml_processor_tool_installation || die
    wget "https://download.fortanix.com/sgx-detect/ubuntu"$ubuntu_version"/sgx-detect" -P $TMP_DIR || die
    chmod +x $TMP_DIR/sgx-detect
    $TMP_DIR/sgx-detect --export > $TMP_DIR/sgx_detect.yaml
    sed -i -e '1,2d' $TMP_DIR/sgx_detect.yaml
}

function check_sgx_support() {
    if [ ! -f $TMP_DIR/sgx_detect.yaml ]; then
        generate_sgx_detect_report
    fi
    local cpu_sgx_support=$(yq .cpuid_7h.Ok.sgx $TMP_DIR/sgx_detect.yaml)
    if [ "$cpu_sgx_support" == "true" ]; then
        echo "* The hardware platform does have SGX support" >&3
    else
        echo "* The hardware platform does not have SGX support" >&3
        die
    fi
}

function check_dcap_provider_library_installation() {
    echo "* Checking If DCAP Provider library is installed" >&3
    local dcap_provider_lib_file=$(find /usr/lib -iname "libdcap_quoteprov.so*")
    if [[ $dcap_provider_lib_file == *"libdcap_quoteprov.so"* ]]; then
        dcap_provider_library_installed=true
        echo "* DCAP Provider library is installed on the node" >&3
    else
        dcap_provider_library_installed=false
        echo "* DCAP Provider library is not installed on the node" >&3
    fi
}

function check_flc_support() {
    echo "* Checking If FLC is supported" >&3
    if [ ! -f $TMP_DIR/sgx_detect.yaml ]; then
        generate_sgx_detect_report
    fi
    local cpu_sgx_lc_support=$(yq .cpuid_7h.Ok.sgx_lc $TMP_DIR/sgx_detect.yaml)
    local bios_sgx_lc_support=$(yq .msr_3ah.Ok.sgx_lc $TMP_DIR/sgx_detect.yaml)
    local bios_sgx_lc_locked=$(yq .msr_3ah.Ok.locked $TMP_DIR/sgx_detect.yaml)

    if [ "$cpu_sgx_lc_support" == "true" ]  && [ "$bios_sgx_lc_support" == "true" ] && [ "$bios_sgx_lc_locked" == "true" ]; then
        flc_supported=true
        echo "* Flexible Launch Control is supported on the node" >&3
    else
        flc_supported=false
        echo "* Flexible Launch Control is not supported on the node" >&3
    fi
}

function check_dcap_attestation_support() {
    check_dcap_provider_library_installation || die
    check_flc_support || die
    if [ "$flc_supported" == "true" ]  && [ "$dcap_provider_library_installed" == "true" ]; then
        dcap_attestation_supported=true
    else
        dcap_attestation_supported=false
    fi
}

function attestation_type() {
    if [[ -z "${attestation_type}" ]]; then
        echo "* Attestation type is not passed as parameter" >&3
        echo "* Using EPID as attestation type by default" >&3
        attestation_type="EPID"
    else
        echo "* Attestation type $attestation_type is passed as parameter" >&3
        if [ "$attestation_type" == "DCAP" ]; then
            check_dcap_attestation_support || die
            if [ "$dcap_attestation_supported" == "true" ]; then
                echo "* DCAP attestation is supported on the node" >&3
            else
                echo "Error: DCAP attestation is not supported on the node but attestation type passed as parameter is DCAP"
                die
            fi
        elif [ "$attestation_type" != "EPID" ]; then
            echo "Error: Invalid Attestation type $attestation_type is passed as parameter. Supported EPID or DCAP"
            die
        fi
    fi
}

function check_if_ubuntu_version_is_supported() {
    if [ "$ubuntu_version" != "16.04" ]  && [ "$ubuntu_version" != "18.04" ] && [ "$ubuntu_version" != "20.04" ] && [ "$ubuntu_version" != "22.04" ]; then
        echo "em-agent is not supported on ubuntu version: $ubuntu_version"
        die
    fi
}

clean() {
    rm -rf $TMP_DIR || true
}

trap clean EXIT




while [ "$1" != "" ]; do
    case $1 in
        --no-enclave-os )            install_enclave_os=false
                                     ;;
        -h | --help )                usage
                                     exit
                                     ;;
        --attestation-type=* )       attestation_type=$(echo "$1" | cut -f2 -d=)
                                     ;;
        * )                  api_key="$1"
    esac
    shift
done

if [ -z "$api_key" ]; then
    join_token_missing_msg="Error: join token argument is missing. The join token is the way by which this node will authenticate itself to Enclave Manager. Please consult the INSTALLER_README.md file for how to get a join token from Enclave Manager."
    echo -e $join_token_missing_msg
    usage
    exit 1
fi

set -u
log_file=$(mktemp /var/log/node_agent_installer.XXXXXXXXX)
echo "This installation will be logged at $log_file"
exec 3>&1 4>&2
trap 'exec 2>&4 1>&3' 0 1 2 3
exec 1>"$log_file" 2>&1
set -x

# Try to read the API_KEY_PATH from /etc/em-agent/em-agent.conf if that file already exists from a previous
# installation. The em-agent.deb package won't overwrite that config file if it already exists.
if [ -f "$em_agent_conf_file" ]; then
    source "$em_agent_conf_file"
fi
api_key_path="${API_KEY_PATH:-$default_api_key_path}"

echo -n "* Installing join token to $api_key_path... " >&3
mkdir -p "$(dirname "$api_key_path")" || die
echo "$api_key" > "$api_key_path" || die
echo "done" >&3

check_if_ubuntu_version_is_supported || die
check_sgx_support || die

# On ubuntu22.04 use packages from focal repository of fortanix for now
if [ "$ubuntu_version" == "22.04" ]; then
    ubuntu_codename=focal
fi
echo -n "* Installing intel-sgx apt repository... " >&3
append_line_if_not_exists "deb [arch=amd64] https://download.01.org/intel-sgx/sgx_repo/ubuntu $ubuntu_codename main" "/etc/apt/sources.list.d/intel-sgx.list" || die
curl -sSL "https://download.01.org/intel-sgx/sgx_repo/ubuntu/intel-sgx-deb.key" | apt-key add - || die
echo "done" >&3

echo -n "* Installing Fortanix apt repository... " >&3
append_line_if_not_exists "deb [arch=amd64] https://download.fortanix.com/linux/apt $ubuntu_codename main" "/etc/apt/sources.list.d/fortanix.list"  || die
curl -sSL "https://download.fortanix.com/linux/apt/fortanix.gpg" | apt-key add - || die
echo "done" >&3

echo -n "* Updating package list... " >&3
apt-get update || die
echo "done" >&3

attestation_type || die

attestation_type_path="${ATTESTATION_TYPE_PATH:-$default_attestation_type_path}"

echo -n "* Installing attestation type to $attestation_type_path... " >&3
mkdir -p "$(dirname "$attestation_type_path")" || die
echo "$attestation_type" > "$attestation_type_path" || die
echo "done" >&3

install_packages_msg="* Installing em-agent"
packages_to_install=("em-agent")

if [ "$install_enclave_os" = true ]; then
    FLAGS=$(LD_SHOW_AUXV=1 sleep 0 | { grep -Po "AT_HWCAP2:[[:space:]]*\K([0-9x]*)" || true; })
    FSGSBASE=$(((FLAGS & 2)/2))

    install_packages_msg+=" and "enclave-os-host""
    packages_to_install+=("enclave-os-host") 
        
    
fi

install_packages_msg="$install_packages_msg... "

if [[ $attestation_type == 'EPID' ]]; then
    echo -n "* Installing sgx-support-epid... " >&3
    apt-get install -y sgx-support-epid || die
    echo "done" >&3
else
    echo -n "* Installing sgx-support-dcap... " >&3
    apt-get install -y sgx-support-dcap || die
    echo "done" >&3
fi

echo -n "$install_packages_msg" >&3
apt-get install -y "${packages_to_install[@]}" || die
echo "done" >&3

echo "Installation finished" >&3
